;					TETRIS
;			(C) Rickard Gune 1998
;
;	This is shareware, use it at your own risk.
;
;	You will find more info about the project
;	and more info on video signals here:
;	http://www.rickard.gunee.com/projects
;
;	Outputs composite NTSC video signal using
;	only two resistors!


	include	P16F628A.INC
	__CONFIG 0x3F2A

startspeed	equ	0x18
movespeed	equ	0x06

up1b		equ	3
down1b		equ	2
left1b		equ	5
right1b		equ	4
fire1b		equ	1
up2b		equ	7
down2b		equ	6
left2b		equ	2
right2b		equ	3
fire2b		equ	1
up1p		equ	PORTB
down1p		equ	PORTB
left1p		equ	PORTB
right1p		equ	PORTB
fire1p		equ	PORTB
up2p		equ	PORTB
down2p		equ	PORTB
left2p		equ	PORTA
right2p		equ	PORTA
fire2p		equ	PORTA

counter0	equ	0x20
counter1	equ	0x21
counter2	equ	0x22
counter3	equ	0x23
nextblocktyp	equ	0x24
blockx		equ	0x25
blocky		equ	0x26
blocktyp	equ	0x27
line		equ	0x28
x			equ	0x29
y			equ	0x2A
delaycnt	equ	0x2B
angle		equ	0x2C
blockstuff	equ	0x2D
fallcnt		equ	0x2E
points		equ	0x2F
random		equ	0x32	
stuff		equ	0x33
m_freq		equ	0x34
m_cnt		equ	0x35
m_songcnt	equ	0x36

buffer		equ	0x38
currbl		equ	0x58
x0			equ	0x60
y0			equ	0x61
movecnt		equ	0x62
remline		equ	0x63

hsfall		equ	0
rotate		equ	1
goleft		equ	2
goright		equ	3
drop		equ	4
rotat		equ	5
gameover	equ	5

delay		MACRO
		LOCAL	label
		movwf	delaycnt
label		decfsz	delaycnt,F
		goto label
		ENDM

dnop		MACRO
		LOCAL	label
label		goto	label+1
		ENDM

		org 0x000
		bcf	STATUS,RP0
		bcf	STATUS,RP1
		MOVLW 0x07	 ;Turn comparators off and
		MOVWF CMCON	;enable pins for I/O
		goto inittetris


;------------ This table contains the 3 note lengthes for the 5 speeds --------

getlength	addwf	PCL,F
		retlw	0x0B
		retlw	0x16
		retlw	0x1D
		retlw	0x09
		retlw	0x12
		retlw	0x19
		retlw	0x07
		retlw	0x0D
		retlw	0x11
		retlw	0x04
		retlw	0x08
		retlw	0x0C
		retlw	0x02
		retlw	0x04
		retlw	0x06

;------------------------ set bit in the gamefield ----------------------------

setbit		call	getbit		;get bitbyte and bitmask	20 cycles
		iorwf	INDF,F 		;set bit
		return

;----------------------- clear bit in the gamefield ---------------------------

clrbit		call	getbit		;get bitbyte and bitmask	21 cycles
		xorlw	0xff		;invert bitmask
		andwf	INDF,F		;clear bit
		return

;-------------------- point at byte, and return bitmask -----------------------

getbit		movlw	buffer 		;15 cycles
		btfsc	x,3
		movlw	buffer+1
		clrc
		rlf	y,F
		addwf	y,W
		movwf	FSR			;FSR = 2*y + x<<3 + buffer
		movfw	x
		andlw	7			;W = x&7
bitmask		addwf	PCL,F
		retlw	0x01
		retlw	0x02
		retlw	0x04
		retlw	0x08
		retlw	0x10
		retlw	0x20
		retlw	0x40
		retlw	0x80

;------------------------ blocks in compressed format -------------------------

blocks		andlw	0xF
		addwf	PCL,F
		retlw	0x50
		retlw	0x44
		retlw	0xD0
		retlw	0x0C
		retlw	0xD0
		retlw	0x0C
		retlw	0xD0
		retlw	0x3C
		retlw	0xD0
		retlw	0xCC
		retlw	0xF4
		retlw	0xC0
		retlw	0x5C
		retlw	0xC0
		retlw	0x00
		retlw	0x6C

;---------------- convert from compressed format to uncompressed --------------

convert		andlw	3
		addwf	PCL,F
		retlw	0
		retlw	1
		retlw	2
		retlw	-1

;------------------------- generate numbers for points ------------------------

chars		movwf	delaycnt	;13 cycles
		clrc
		rlf	delaycnt,F
		rlf	delaycnt,F
		rlf	delaycnt,F
		movfw	delaycnt
		addwf	line,W
		addwf	PCL,F
;number  0 
		retlw 0x1C
		retlw 0x36
		retlw 0x63
		retlw 0x6B
		retlw 0x63
		retlw 0x36
		retlw 0x1C
		retlw 0x0
;number  1 
		retlw 0x18
		retlw 0x1C
		retlw 0x18
		retlw 0x18
		retlw 0x18
		retlw 0x18
		retlw 0x7E
		retlw 0x0
;number  2 
		retlw 0x3E
		retlw 0x63
		retlw 0x60
		retlw 0x38
		retlw 0xC
		retlw 0x66
		retlw 0x7F
		retlw 0x0
;number  3 
		retlw 0x3E
		retlw 0x63
		retlw 0x60
		retlw 0x3C
		retlw 0x60
		retlw 0x63
		retlw 0x3E
		retlw 0x0
;number  4 
		retlw 0x38
		retlw 0x3C
		retlw 0x36
		retlw 0x33
		retlw 0x7F
		retlw 0x30
		retlw 0x78
		retlw 0x0
;number  5 
		retlw 0x7F
		retlw 0x3
		retlw 0x3
		retlw 0x3F
		retlw 0x60
		retlw 0x63
		retlw 0x3E
		retlw 0x0
;number  6 
		retlw 0x1C
		retlw 0x6
		retlw 0x3
		retlw 0x3F
		retlw 0x63
		retlw 0x63
		retlw 0x3E
		retlw 0x0
;number  7 
		retlw 0x7F
		retlw 0x63
		retlw 0x30
		retlw 0x18
		retlw 0xC
		retlw 0xC
		retlw 0xC
		retlw 0x0
;number  8 
		retlw 0x3E
		retlw 0x63
		retlw 0x63
		retlw 0x3E
		retlw 0x63
		retlw 0x63
		retlw 0x3E
		retlw 0x0
;number  9 
		retlw 0x3E
		retlw 0x63
		retlw 0x63
		retlw 0x7E
		retlw 0x60
		retlw 0x30
		retlw 0x1E
		retlw 0x0

;-------------------------- vertical syncing stuff ----------------------------

shortsync	movwf	counter1
shortsync_l0	bcf	PORTA,0		;2us sync
		bcf	PORTB,0
		dnop
		movlw	0x1D - 5	;30us black
		movwf	counter2
		nop
		bsf	PORTA,0
shortsync_l1	decfsz	counter2,F
		goto	shortsync_l1
		call	sound
		decfsz	counter1,F
		goto	shortsync_l0
		retlw	5

vertsync	movlw	5
		btfss	stuff,7
		movlw	6
		call	shortsync

		btfss	stuff,7
		goto	opage
		bcf	stuff,7
opager

longsync	movwf	counter1
longsync_l0	movlw	0x1D - 5
		movwf	counter2
		bcf	PORTA,0		;30 us sync
		bcf	PORTB,0	
longsync_l1	decfsz	counter2,F
		goto	longsync_l1
		nop			;2us black
		call	sound
		bsf	PORTB,0
		nop
		decfsz	counter1,F
		goto	longsync_l0

		movlw	5
		btfss	stuff,7
		movlw	4
		call	shortsync
		return

opage		bsf	stuff,7	
		goto	opager

;----------------- routines for displaying whole empty lines ------------------

emptylines	movwf	counter1
		dnop
		nop
ell		bcf	PORTA,0
		dnop
ell2		nop
		movlw	2
		delay
		movlw	10
		bsf	PORTA,0
		delay
		call	sound
		movlw	0x20
		delay
		call	sound
		nop
		decfsz	counter1,F
		goto	ell

		movlw	3
		bcf	PORTA,0
		delay
		nop
		movlw	0x10
		bsf	PORTA,0
		delay
		call	sound
		movlw	0x1D
		delay
		dnop
		call	sound
		return

;-- routine for testing if it is possible to place the block on position x,y --

test		movlw	4			;131  cycles
		movwf	counter2		;hitcounter = 4
		movwf	counter0		;test 4 bits
		movlw	currbl			;point at current block
		movwf	FSR			;with FSR
testl		movfw	blockx
		addwf	INDF,W
		incf	FSR,F
		movwf	x			;x = x0 + blockdiffx
		movfw	blocky
		addwf	INDF,W
		incf	FSR,F
		movwf	y			;y = y0 + blockdiffy
		movfw	FSR
		movwf	counter1		;save FSR
		call	getbit			;get bit
		andwf	INDF,W			;test bit
		skpnz
		decf	counter0,F
		movfw	counter1		;restore FSR
		movwf	FSR
		decfsz	counter2,F
		goto	testl
		movf	counter0,F		;set Z if ok
		return

;################### HERE STARTS THE MAIN THREAD OF TETRIS ####################

tetrismain	call	vertsync		;do vertical retrace

		btfsc	stuff,gameover
		goto	nogame

		call	createnext		;create the preview image		(line 0)
		call	remblock		;hide the preview image			(line 1)
		call	createcurrent		;create the current image		(line 2)
		call	remblock		;hide the current image			(line 3)

;----------------------------- make block fall -------------------------------		

		call	hsync			;horizontal sync			(line 4)
		btfsc	blockstuff,hsfall	;check for high speed fall (drop)
		goto	fall
		decfsz	fallcnt,F			;decrease fallcounter
		goto	nofall			;if not zero, then dont fall
fall
		movfw	points			;calculate speed from the most
		sublw	0xa			;significant digit of the score
		movwf	fallcnt
		rlf	fallcnt,F
		rlf	fallcnt,F
		
		incf	blocky,F			;move block down

		call	test			;check if it was ok
		skpnz
		goto	fallok			;if ok, skip the newblock below
		movlw	0x0B
		delay

		decf	blocky,F
		call	showblock		;make the old block visible			(line 5)
		call	hsync			;horizontal sync			(line 6)
newblock	clrf	blockstuff
		movlw	0x9
		movwf	blockx			;blockx = 9
		movlw	0x1
		movwf	blocky			;blocky = 1
		movfw	nextblocktyp
		movwf	blocktyp		;blocktyp = nexttyp
		movfw	random
		movwf	nextblocktyp		;nexttyp = random
		clrf	angle			;angle = 0
		call	incpoints		;add one point

		bcf	blockstuff,hsfall
		movfw	y0
		sublw	1
		skpnz				;if y = 1 then
		bsf	stuff,gameover		;game over

nofallret	

;----------------------------- joystick motion --------------------------------

		btfss	left1p,left1b		;check left
		bsf	blockstuff,goleft
		btfss	right1p,right1b		;check right
		bsf	blockstuff,goright
		btfss	fire1p,fire1b		;check fire (rotate)
		bsf	blockstuff,rotate
		btfss	down1p,down1b		;check down (drop)
		bsf	blockstuff,drop

		btfss	stuff,1			;rotate last time ?
		goto	norotate		;yes, wait for release
		btfsc	blockstuff,rotate	;check rotate
		bsf	blockstuff,rotat	;if rotate start rotate
		btfsc	blockstuff,rotate	;check rotate
		bcf	stuff,1			;if rotation then remember it
		nop
norotater	bcf	blockstuff,rotate	;clear rotation indicator


		btfss	stuff,2			;down last time ?
		goto	nofallu			;yes, wait for release
		btfsc	blockstuff,drop		;check down
		bsf	blockstuff,hsfall	;if down start fall
		btfsc	blockstuff,drop		;check down
		bcf	stuff,2			;if down then remember it
		nop
nofallur	bcf	blockstuff,drop		;and clear down indicator

		movlw	0x23			;wait 33us for line to end
		delay
		nop

;--------------------------- handle rotation ----------------------------------

		btfss	blockstuff,rotat	;rotate ?
		goto	norot			;no, skip all rotation stuff
		incf	angle,F			;try to increase angle
		call	createcurrent		;make block image for this angle	(line 7)
		call	hsync			
		call	test			;ok ?
		skpz				
		decf	angle,F			;no undo angle change
		movlw	0xC			
		delay
		incf	random,F
		movlw	7
		andwf	random,F
		dnop
		call	createcurrent		;create block 				(line 8)
norotr		call	hsync			;					(line 9)
		bcf	blockstuff,rotat	;rotate one step only
		bcf	blockstuff,rotate

;---------------------- move block, left and right ----------------------------

		btfsc	blockstuff,goleft
		decf	blockx,F
		btfsc	blockstuff,goright
		incf	blockx,F
		movlw	0x38
		delay

		call	createcurrent		;create current block image	(line 10)
		call	hsync			;horizontal sync		(line 11)
		call	test
		skpnz
		goto	moveok			;if ok skip restore
		dnop
		dnop
		dnop
nomove		btfsc	blockstuff,goleft
		incf	blockx,F
		btfsc	blockstuff,goright
		decf	blockx,F
		movlw	0x2e
		dnop
		dnop
moveokret	movlw	0x8
		delay


		bcf	blockstuff,goleft	;clear move left indicator
		bcf	blockstuff,goright	;clear move right indicator

;---------------- search for filled line in the 8 first lines ---------------

		bcf	PORTA,0		;start sync				(line 12)
		dnop
		dnop
		movlw	8		;check the first 8 lines this scanline
		movwf	counter0
		movlw	buffer		;set up pointer to screen buffer
		movwf	FSR		
		clrf	line
		dnop
		bsf	PORTA,0		;end sync

		movlw	6		;wait for 6 uS
		delay
		clrf	remline

remfilledl11	movfw	INDF		;get first byte of line
		incf	FSR,F		;move pointer to next byte
		sublw	0xF0		;is it "filled" ?
		skpz
		goto	nofilled11	;nope, we're outa here
		movfw	INDF		;get second byte of line
		incf	FSR,F		;move pointer to next byte (next line)
		sublw	0xFF		;is it "filled" ?
		skpz
		goto	nofilled21	;nope, we're outa here
		movfw	line
		movwf	remline		;save line nr
		nop
nofilledr1	incf	line,F		;count up line nr
		decfsz	counter0,F
		goto	remfilledl11

;------------ search for filled line in the following 7 lines -------------

		bcf	PORTA,0		;start sync				(line 13)
		movlw	3
		delay
		movlw	7		;check next 7 lines this scanline
		bsf	PORTA,0		;end sync
		movwf	counter0

remfilledl12	movfw	INDF		;get first byte of line
		incf	FSR,F		;move pointer to next byte
		sublw	0xF0		;is it "filled" ?
		skpz
		goto	nofilled12	;nope, we're outa here
		movfw	INDF		;get second byte of line
		incf	FSR,F		;move pointer to next byte (next line)
		sublw	0xFF		;is it "filled" ?
		skpz
		goto	nofilled22	;nope, we're outa here
		movfw	line
		movwf	remline		;save line nr
		nop
nofilledr2	incf	line,F		;count up line nr
		decfsz	counter0,F
		goto	remfilledl12

		movlw	0x0B		;wait for 12 uS
		delay

; ---------- if one line is going to be removed, add lots of points -----------

		call	hsync			;		(line 14)
		movf	remline,F
		skpnz
		goto	noaddpoints

		call	incpoints
		call	incpoints
		call	incpoints
		call	incpoints
		call	incpoints
		call	incpoints
		call	incpoints

noaddpointsr	

; --------------- remove line part 1: if neccessary, remove 15 blocks ---------

		bcf	PORTA,0		;start sync				(line 15)
		movlw	0x10		;remove all crap at top of screen
		movwf	buffer
		movlw	0x80			
		movwf	buffer + 1
		dnop
		movfw	remline
		addwf	remline,W
		movwf	counter0	;half lines to be removed = 2*remline
		addlw	buffer + 1	;start pointer = 2*remline + buffer base + 1
		movwf	FSR
		bsf	PORTA,0		;end sync

		movlw	0xF
		subwf	counter0,W
		movwf	counter2
		movwf	counter3
		btfsc	counter2,7	;do we only have to move less than 15 blocks ?
		goto	remshort	;yes, skip this part

		movlw	0xF		;else remove 15 lines first
		movwf	counter0

remfilledl21	decf	FSR,F		;move blocks two steps in buffer
		decf	FSR,F
		movfw	INDF		;get data at FSR - 2
		incf	FSR,F		;move back again
		incf	FSR,F
		movwf	INDF		;put it at FSR
		decf	FSR,F		;point to next block to move
		decfsz	counter0,F
		goto	remfilledl21

		movlw	0x7		;wait 7us for line to end
		delay

; --------------- remove line part 2: remove the other blocks -------------
		
remshortr:	bcf	PORTA,0		;start sync				(line 16)
		movlw	3
		delay
		nop
		bsf	PORTA,0		;end sync

		btfsc	counter3,7	;if lines to draw is less than zero
		goto	noremfilled	;then don't draw any lines

remfilledl31	decf	FSR,F		;move blocks two steps in buffer (uses counter1 * 10)
		decf	FSR,F
		movfw	INDF		;get data at FSR - 2
		incf	FSR,F
		incf	FSR,F
		movwf	INDF		;put it at FSR
		decf	FSR,F		;FSR++ 
		decfsz	counter2,F
		goto	remfilledl31

		movfw	counter3	;calc wait time
		sublw	0xF + 2
		movwf	counter1

remfilledl32	movlw	2		;wait for ((15+2)-counter3)*10 cycles
		delay
		decfsz	counter1,F
		goto	remfilledl32

		dnop
		dnop
		dnop
noremfilledr	

;-------------------------- some block image management -----------------------

		call	createcurrent						;(line 17)
		call	showblock						;(line 18)
		call	createnext						;(line 19)
		call	showblock						;(line 20)

;---------------------- start drawing the graphics on screen ------------------

nogamer		movlw	0x10 			;empty lines at top of screen
		movwf	counter1
		bcf	PORTA,0
		call	ell2
		movlw	0x0
		tris	PORTB
		clrf 	PORTB
		movlw	buffer			;set up pointer to buffer
		movwf	FSR
		movlw	0x10			;16 blocklines (each 11+2 lines)
		movwf	counter2
tml0		movlw	0x9			;9 lines for each block
		movwf	counter3
tml1		bcf	PORTA,0			;start sync
		movlw	3			;keep low for 4us
		delay	
		nop
		bsf	PORTA,0

		nop				;colorburst + black  (15.67 us)
		movlw	0x0F - 5 
		delay
		call	sound

		movfw	INDF			;get left byte of gfx
		movwf	PORTB			;start showing first bit
		movlw	7			;set up counter for next 7 bits
		movwf	counter0
		incf	FSR,F			;point at right gfx byte
db0		bcf	PORTB,0			;one black vertical line between the blocks
		rrf	PORTB,F			;show next bit
		decfsz	counter0,F		;keep on looping, showing all bits
		goto	db0
		movfw	INDF			;get right byte of gfx
		bcf	PORTB,0			;one black vertical line between the blocks
		movwf	PORTB			;start showing first bit
		movlw	7			;set up counter for next 7 bits
		movwf	counter0
		decf	FSR,F			;point at left gfxbyte again
db1		bcf	PORTB,0			;one black vertical line between the blocks
		rrf	PORTB,F			;show next bit
		decfsz	counter0,F		;keep on looping, showing all bits
		goto	db1
		movlw	0xF - 5
		bcf	PORTB,0			;set level to black
		delay				;for 16us
		call	sound
		nop
		movlw	2			;prepare for 1 emptyline
		movwf	counter1
		decfsz	counter3,F
		goto	tml1			;do all 9 lines
		bcf	PORTA,0
		call	ell2			;do 2 emptylines
		dnop
		incf	FSR,F			;point at next line (each line is 2 bytes)
		incf	FSR,F
		decfsz	counter2,F		;do all 16 blocklines
		goto	tml0

;------------------------------- display score --------------------------------

		clrf	line
		call	hsync

		movlw	0x39
		delay
		nop
		movlw	8
		movwf	counter2
shpl		call	hsync
		movlw	0x18
		delay
		call	showp
		movlw	4
		delay
		call	hsync
		movlw	0x18
		delay
		call	showp
		movlw	2
		delay
		dnop
		incf	line,F
		decfsz	counter2,F
		goto	shpl

;--------------- some empty lines, and also some random stuff -----------------

		movlw	0x1C   		;empty lines at bottom of screen
		movwf	counter1
		call	ell

		call	music

		movlw	1
		btfsc	stuff,gameover
		movwf	m_freq

		movlw	0xFE
		tris	PORTB

		incf	random,F
		movlw	7
		andwf	random,F

		movf	movecnt,F
		skpz
		decf	movecnt,F		
		goto	tetrismain		;next frame

;################ THIS IS THE END OF THE MAIN THREAD OF TETRIS ################


;--------------------------- create next block stuff --------------------------

createnext	bcf	PORTA,0			;start sync
		movfw	blocktyp
		movwf	counter3		;save current block kind
		movfw	nextblocktyp
		movwf	blocktyp		;set the next block kind
		movfw	angle
		movwf	delaycnt
		clrf	angle
		movlw	1			;set x and ypos, they
		movwf	x0			;are not used here,
		movlw	2			;but we will need them later
		movwf	y0
		bsf	PORTA,0			;end sync
		call	makeblock		;create block
		movfw	counter3
		movwf	blocktyp
		movfw	delaycnt
		movwf	angle
		return

;------------------ create current block stuff scan line ---------------------

createcurrent	bcf	PORTA,0			;start sync
		movlw	0x2
		delay
		movfw	blockx			;x0 anf y0 are not
		movwf	x0			;used here, but we will use
		movfw	blocky			;them later
		movwf	y0
		bsf	PORTA,0			;end sync
		call	makeblock
		return

;------------------------ remove big block scan line -------------------------

remblock	bcf	PORTA,0			;start sync
		movlw	3
		delay
		nop
		bsf	PORTA,0			;end sync
		movlw	4
		movwf	counter0
		movlw	currbl
		movwf	FSR
hbl		movfw	x0
		addwf	INDF,W
		incf	FSR,F
		movwf	x		;x = x0 + relx
		movfw	y0
		addwf	INDF,W
		incf	FSR,F
		movwf	y		;y = y0 + rely
		movfw	FSR
		movwf	counter1
		call	clrbit
		movfw	counter1
		movwf	FSR
		decfsz	counter0,F
		goto	hbl
		movlw	9
		delay
		return

;--------------------------- show block scan line ----------------------------

showblock	bcf	PORTA,0			;start sync
		movlw	3
		delay
		movlw	4
		bsf	PORTA,0			;end sync
		movwf	counter0
		movlw	currbl
		movwf	FSR
sbl		movfw	x0
		addwf	INDF,W
		incf	FSR,F
		movwf	x			;x = x0 + relx
		movfw	y0
		addwf	INDF,W
		incf	FSR,F
		movwf	y		;y = y0 + rely
		movfw	FSR
		movwf	counter1
		call	setbit
		movfw	counter1
		movwf	FSR
		decfsz	counter0,F
		goto	sbl
		movlw	0xA
		delay
		return

;------------------------- output one sync pulse ------------------------------
;17 cycles

hsync		bcf	PORTA,0
		movlw	3
		delay
		nop
		bsf	PORTA,0			;end sync
		return

;-------------------------- add one point to score ----------------------------

incpoints	incf	points + 2,F		;25 cycles
		movfw	points + 2
		sublw	0xA
		skpz
		goto	firstok
		clrf	points + 2
		incf	points + 1,F
		movfw	points + 1
		sublw	0xA
		skpz
		goto	secondok
		clrf	points + 1
		incf	points,F
		movlw	2
		delay
		return

firstok		nop
		dnop
		dnop
secondok	dnop
		movlw	3
		delay
		return

;------------------------- part of line of score ------------------------------

showp		movfw	points + 2		;89 cycles
		call	chars
		movwf	counter1
		movfw	points + 1
		call	chars
		movwf	counter0
		movfw	points 
		call	chars
		call	shiftout
		movfw	counter0
		call	shiftout
		movfw	counter1
		call	shiftout
		return

;----------- routine used by showp to shift out one byte to display -----------

shiftout	movwf	PORTB		;13 cycles
		rrf	PORTB,F
		rrf	PORTB,F
		rrf	PORTB,F
		rrf	PORTB,F
		rrf	PORTB,F
		rrf	PORTB,F
		rrf	PORTB,F
		bcf	PORTB,0
		return

;-------- makeblock creates a block of blocktype into the image buffer -------
;167 cycles

makeblock	
		movlw	3			;keep angle 0-3
		andwf	angle,F
		movf	blocktyp,F		;if blocktype = 0
		skpnz
		clrf	angle			;then dont allow rotation

		bcf	stuff,3			;clear x-mirror flag
		bcf	stuff,4			;clear y-mirror flag
		btfsc	angle,1			;if angle = 2 or 3
		bsf	stuff,4			;then set y-mirror flag
		movfw	angle
		xorlw	1			;if angle = 1
		skpnz
		bsf	stuff,3			;then set x-mirror flag
		xorlw	1+2			;if angle = 2
		skpnz
		bsf	stuff,3			;then set x-mirror flag
		clrc
		rlf	blocktyp,W		;pointer = blocktyp * 2
		call	blocks			;get x-blockinfo from pointer
		movwf	counter0		;save x-blockinfo
		clrc
		rlf	blocktyp,W
		addlw	1			;pointer = blocktyp * 2 + 1
		call	blocks			;get y-blockinfo from pointer
		movwf	counter1		;save y-blockinfo
		btfss	angle,0			;if angle = 1, or angle = 3
		goto	mbnoswap
		xorwf	counter0,W		;then swap x and y blockinfo
		xorwf	counter1,F
		xorwf	counter0,F
mbnoswapr	movlw	4
		movwf	counter2		;we have 4 coordinat pairs per block
		movlw	currbl
		movwf	FSR			;setup pointer to image buffer
mbl0		movfw	counter0		;get x-blockinfo
		call	convert			;convert the two first bits into a coordinat value
		btfsc	stuff,3			;if x-mirror flag = 1
		sublw	0			;then make the x-coordinate negative
		movwf	INDF			;save x-coordinate in image buffer
		incf	FSR,F			;move pointer to next position in image buffer
		movfw	counter1		;get y-block info
		call	convert			;convert the two first bits into a coordinate value
		btfsc	stuff,4			;if y-mirror flag = 1
		sublw	0			;then make the y-coordinate negative
		movwf	INDF			;save y-coordinate in image buffer
		incf	FSR,F			;move pointer to next position in image buffer
		rrf	counter0,F		;rotate x-blockinfo to the next two bits
		rrf	counter0,F
		rrf	counter1,F		;rotate y-blockinfo to the next two bits
		rrf	counter1,F
		decfsz	counter2,F		;do all 4 coordinate pairs
		goto	mbl0
		return

;--------------------- initialize tetris, clear stuff -------------------------

inittetris	movlw	0x21
		movwf	FSR
		movlw	0x43
		movwf	counter0
cllp		clrf	INDF
		incf	FSR,F
		decfsz	counter0,F
		goto	cllp

		movlw	buffer			;point to buffer
		movwf	FSR
		movlw	0xf			;15 blocklines
		movwf	counter0
itl0		movlw	0x10			;left block: ....*...
		movwf	INDF
		incf	FSR,F			;point to next block
		movlw	0x80			;right block: .......*
		movwf	INDF
		incf	FSR,F			;point to next block
		decfsz	counter0,F		;do all 15 lines
		goto	itl0
		movlw	0xf0			;bottom left ....****
		movwf	buffer + 0x1E
		movlw	0xff			;bottom right ********
		movwf	buffer + 0x1F
		movlw	0xE
		tris	PORTA
		movlw	0xFE
		tris	PORTB

		movlw	1
		movwf	fallcnt			;fallcount = 1
		movwf	movecnt			;move count = 1
		movwf	m_songcnt		;song count = 1
		clrf	EEADR
		call	createcurrent
		goto	newblock

;------------------------------ delay stuff -----------------------------------

noaddpoints	movlw	0x39
		delay
		goto	noaddpointsr

nofall		movlw	0x2D
		delay
		dnop

fallok		movlw	0xB
		delay
		call	hsync
		movlw	0x3A
		delay
		call	hsync
		movlw	0xC
		delay
		nop
		goto	nofallret

moveok		nop
		movf	movecnt,F
		skpz
		goto	nomove
		movlw	movespeed
		movwf	movecnt

		incf	random,F
		movlw	7
		andwf	random,F
		dnop
		goto	moveokret

nofilled11	incf	FSR,F
		nop
		dnop
nofilled21	goto	nofilledr1

nofilled12	incf	FSR,F
		nop
		dnop
nofilled22	goto	nofilledr2

remshort	movlw	0x36		;wait 56us for sync to end
		delay
		dnop
		movfw	counter0
		movwf	counter2
		movwf	counter3
		skpnz
		bsf	counter3,7
		goto	remshortr	;then continue removing lines

noremfilled	movlw	0x39		;wait 59us for sync to end
		delay
		dnop
		goto	noremfilledr		;no lines removed, continue and stuff..

norot		call	createcurrent
		call	createcurrent
		call	hsync
		movlw	0x39
		delay
		nop
		goto	norotr

nofallu		btfss	blockstuff,drop
		bsf	stuff,2
		goto	nofallur

norotate	btfss	blockstuff,rotate
		bsf	stuff,1
		goto	norotater

mbnoswap	goto	mbnoswapr

nogame		call	hsync
		btfss	fire1p,fire1b
		goto	inittetris
		movlw	0x14
		call	emptylines
		goto	nogamer


;---------------------------- Sound output routine  ---------------------------
;15 cycles

sound		bsf	PORTA,4		;set sound output to 1
		btfss	stuff,6		;check sound state
		bcf	PORTA,4		;if soundstate is 0 then sound output should be 0
		movfw	m_freq		;get current frequency
		decfsz	m_cnt,F		;decrease sound counter, check if it becomes zero
		goto	soundsk
		movwf	m_cnt		;if zero, set music counter to current frequency
		btfsc	stuff,6
		goto	skstuff
		nop
		bsf	stuff,6
		return
skstuff		bcf	stuff,6
		return
soundsk		dnop
		dnop
		return

;------------------------ Set frequency (music routine) -----------------------

music	call	hsync
		movlw	0x36		;prepare for some long delay or something
		decfsz	m_songcnt,F	;update music counter
		goto	nochnote	;if not zero, dont update
		bsf		STATUS,RP0	;set bank1 where eeprom regs are
		bsf		EECON1,RD
		bcf		STATUS,RP0	;set bank0 to use other regs
		movf	m_freq,F		;check current frequency
		skpz
		goto	pause		;if freq is not zero, make a pause

		bsf		STATUS,RP0	;set bank1 where eeprom regs are
		movfw	EEDATA		;get one note
		bcf		STATUS,RP0	;set bank0 to use other regs
		andlw	0x3F		;mask out frequenvy
		movwf	m_freq		;store it
		bsf		STATUS,RP0	;set bank1 where eeprom regs are
		swapf	EEDATA,W	;get note + length, swap upper and lower part into W
		bcf		STATUS,RP0	;set bank0 to use other regs
		andlw	0xC		;mask out length
		movwf	counter3
		clrc
		rrf		counter3,F	;rotate down the length
		rrf		counter3,F
		clrc
		rrf		points,W
		movwf	delaycnt
		clrc
		rlf		delaycnt,F
		addwf	delaycnt,W	;points/2*3
		addwf	counter3,W	;points/2*3 + shortlength

		call	getlength	;get real length
		movwf	m_songcnt
		bsf		STATUS,RP0	;set bank1 where eeprom regs are
		incf	EEADR,F		;next song position
		movfw	EEADR
		sublw	0x34		;end of song ?
		skpnz
		clrf	EEADR		;if so, restart song
		bcf		STATUS,RP0	;set bank0 to use other regs

		movlw	0x29		;this delay is not very critical
nochnote delay
		return


pause		clrf	m_freq	
		movlw	2
		movwf	m_songcnt
		movlw	0x2B + 5
		goto	nochnote


;------------------------- the song: karaboschka ------------------------------

m_d3	equ	0x35		; note definitions
m_e3	equ	0x2F
m_f3	equ	0x2D
m_g3	equ	0x28
m_a3	equ	0x24
m_b3x	equ	0x21
m_c4	equ	0x1E
m_d4	equ	0x1B

m_l4	equ	0x00		; length definition
m_l2	equ	0x40
m_l2x	equ	0x80

		org	0x2100
 
		dw	m_a3  +  m_l2
		dw	m_e3  +  m_l4
		dw	m_f3  +  m_l4
		dw	m_g3  +  m_l2
		dw	m_f3  +  m_l4
		dw	m_e3  +  m_l4
		dw	m_d3  +  m_l2x
		dw	m_f3  +  m_l4
		dw	m_a3  +  m_l2
		dw	m_g3  +  m_l4
		dw	m_f3  +  m_l4
		dw	m_e3  +  m_l2x
		dw	m_f3  +  m_l4
		dw	m_g3  +  m_l2
		dw	m_a3  +  m_l2
		dw	m_f3  +  m_l2
		dw	m_d3  +  m_l2
		dw	m_d3  +  m_l2

		dw	m_b3x +  m_l2x
		dw	m_c4  +  m_l4
		dw	m_d4  +  m_l2
		dw	m_c4  +  m_l4
		dw	m_b3x +  m_l4
		dw	m_a3  +  m_l2x
		dw	m_f3  +  m_l4
		dw	m_a3  +  m_l2
		dw	m_g3  +  m_l4
		dw	m_f3  +  m_l4
		dw	m_e3  +  m_l2x
		dw	m_f3  +  m_l4
		dw	m_g3  +  m_l2
		dw	m_a3  +  m_l2
		dw	m_f3  +  m_l2
		dw	m_d3  +  m_l2
		dw	m_d3  +  m_l2
 
		dw	m_b3x +  m_l2x
		dw	m_c4  +  m_l4
		dw	m_d4  +  m_l2
		dw	m_c4  +  m_l4
		dw	m_b3x +  m_l4
		dw	m_a3  +  m_l2x
		dw	m_f3  +  m_l4
		dw	m_a3  +  m_l2
		dw	m_g3  +  m_l4
		dw	m_f3  +  m_l4
		dw	m_e3  +  m_l2x
		dw	m_f3  +  m_l4
		dw	m_g3  +  m_l2
		dw	m_a3  +  m_l2
		dw	m_f3  +  m_l2
		dw	m_d3  +  m_l2
		dw	m_d3  +  m_l2

		end

